import { useState, useEffect, useMemo } from 'react'
import { OrderService } from 'services'
import { OrdersRequestBody } from 'ManagerRequests'
import { Order } from 'Order'

enum ResStatus {
    IDLE = "IDLE",
    FETCHING = "FETCHING",
    SUCCESS = "SUCCESS",
    ERROR = "ERROR"
}

const initialConstraints: OrdersRequestBody = {
    pagination: { page: 0, size: 15},
    date: { start: null, end: null, filter: null },
    search: { by: 'orderTrackingCode', query: "" },
    status: { received: false, fulfilled: false, cancelled: false }
}


const useOrders = (args : OrdersRequestBody = initialConstraints) => {
    const [status, setStatus] = useState<ResStatus>(ResStatus.IDLE);
    const [dataMap, setDataMap] = useState<{string: Order} | {}>({});
    const [error, setError] = useState(null);
    const [pagFilters, setPagFilters] = useState(args.pagination)
    const [dateFilters, setDateFilters] = useState(args.date)
    const [searchFilters, setSearchFilters] = useState(args.search)
    const [statusFilters, setStatusFilters] = useState(args.status)


    const buildDataMap = (ordersArr: Order[]) => {
        const newDataMap: any = {}
        for (const order of ordersArr)
            newDataMap[order.id] = order;
        return newDataMap
    }


    const fetchOrders = async (requestBody: OrdersRequestBody) => {
        setStatus(ResStatus.FETCHING);
        try {
            const ordersData: Order[] = await OrderService.orders(requestBody)
            setDataMap(prevData => ({ ...prevData, ...buildDataMap(ordersData) }));
            setStatus(ResStatus.SUCCESS)
        } catch (err) {
            setError(err)
            setStatus(ResStatus.ERROR);
        }
    };
    useEffect(() => {
        fetchOrders({
            pagination: pagFilters,
            date: dateFilters,
            search: searchFilters,
            status: statusFilters
        });
    }, [ pagFilters, dateFilters, searchFilters, statusFilters ]);


    const orders: Order[] = useMemo(() => {
       return Object.values(dataMap).sort((o1, o2) => o2.orderUpdatedAt - o1.orderUpdatedAt)
    }, [dataMap])


    return { 
        status, 
        orders,
        error,
        fetchOrders,
        ResStatus,
        pagFilters, setPagFilters,
        dateFilters, setDateFilters,
        searchFilters, setSearchFilters,
        statusFilters, setStatusFilters
    };
};

export default useOrders;